@@ -46,7 +46,7 @@ struct _GtkComboBox
   GtkBin parent_instance;
 
   /*< private >*/
-  GtkComboBoxPrivate *priv;
+  GtkComboBoxPrivate *GSEAL (priv);
 };
 
 struct _GtkComboBoxClass
@@ -67,9 +67,11 @@ struct _GtkComboBoxClass
 
 
 /* construction */
-GType         gtk_combo_box_get_type         (void) G_GNUC_CONST;
-GtkWidget    *gtk_combo_box_new              (void);
-GtkWidget    *gtk_combo_box_new_with_model   (GtkTreeModel    *model);
+GType         gtk_combo_box_get_type                 (void) G_GNUC_CONST;
+GtkWidget    *gtk_combo_box_new                      (void);
+GtkWidget    *gtk_combo_box_new_with_entry           (void);
+GtkWidget    *gtk_combo_box_new_with_model           (GtkTreeModel *model);
+GtkWidget    *gtk_combo_box_new_with_model_and_entry (GtkTreeModel *model);
 
 /* grids */
 gint          gtk_combo_box_get_wrap_width         (GtkComboBox *combo_box);
@@ -112,7 +114,18 @@ GtkTreeViewRowSeparatorFunc gtk_combo_box_get_row_separator_func (GtkComboBox
 void                        gtk_combo_box_set_row_separator_func (GtkComboBox                *combo_box,
 								  GtkTreeViewRowSeparatorFunc func,
 								  gpointer                    data,
-								  GtkDestroyNotify            destroy);
+								  GDestroyNotify              destroy);
+
+void               gtk_combo_box_set_button_sensitivity (GtkComboBox        *combo_box,
+							 GtkSensitivityType  sensitivity);
+GtkSensitivityType gtk_combo_box_get_button_sensitivity (GtkComboBox        *combo_box);
+
+gboolean           gtk_combo_box_get_has_entry          (GtkComboBox        *combo_box);
+void               gtk_combo_box_set_entry_text_column  (GtkComboBox        *combo_box,
+							 gint                text_column);
+gint               gtk_combo_box_get_entry_text_column  (GtkComboBox        *combo_box);
+
+#if !defined (GTK_DISABLE_DEPRECATED) || defined (GTK_COMPILATION)
 
 /* convenience -- text */
 GtkWidget    *gtk_combo_box_new_text         (void);
@@ -127,13 +140,13 @@ void          gtk_combo_box_remove_text      (GtkComboBox     *combo_box,
                                               gint             position);
 gchar        *gtk_combo_box_get_active_text  (GtkComboBox     *combo_box);
 
+#endif
+
 /* programmatic control */
 void          gtk_combo_box_popup            (GtkComboBox     *combo_box);
 void          gtk_combo_box_popdown          (GtkComboBox     *combo_box);
 AtkObject*    gtk_combo_box_get_popup_accessible (GtkComboBox *combo_box);
 
-/* private */
-gboolean     _gtk_combo_box_editing_canceled (GtkComboBox     *combo_box);
 
 G_END_DECLS
 
