@@ -21,9 +21,13 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_MAIN_H__
 #define __GTK_MAIN_H__
 
@@ -35,9 +39,7 @@
 #include <gtk/gtkwindow.h>
 #endif
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 /* Priorities for redrawing and resizing
  */
@@ -78,9 +80,9 @@ GTKMAIN_C_VAR const guint gtk_minor_version;
 GTKMAIN_C_VAR const guint gtk_micro_version;
 GTKMAIN_C_VAR const guint gtk_binary_age;
 GTKMAIN_C_VAR const guint gtk_interface_age;
-gchar* gtk_check_version (guint	required_major,
-			  guint	required_minor,
-			  guint	required_micro);
+const gchar* gtk_check_version (guint	required_major,
+			        guint	required_minor,
+			        guint	required_micro);
 
 
 /* Initialization, exit, mainloop and miscellaneous routines
@@ -97,9 +99,9 @@ gboolean gtk_init_check           (int    *argc,
   
 gboolean gtk_init_with_args       (int            *argc,
 				   char         ***argv,
-				   char           *parameter_string,
+				   const char     *parameter_string,
 				   GOptionEntry   *entries,
-				   char           *translation_domain,
+				   const char     *translation_domain,
 				   GError        **error);
 
 GOptionGroup *gtk_get_option_group (gboolean open_default_display);
@@ -154,6 +156,7 @@ void	   gtk_grab_add		   (GtkWidget	       *widget);
 GtkWidget* gtk_grab_get_current	   (void);
 void	   gtk_grab_remove	   (GtkWidget	       *widget);
 
+#if !defined (GTK_DISABLE_DEPRECATED) || defined (GTK_COMPILATION)
 void	   gtk_init_add		   (GtkFunction	       function,
 				    gpointer	       data);
 void	   gtk_quit_add_destroy	   (guint	       main_level,
@@ -165,10 +168,9 @@ guint	   gtk_quit_add_full	   (guint	       main_level,
 				    GtkFunction	       function,
 				    GtkCallbackMarshal marshal,
 				    gpointer	       data,
-				    GtkDestroyNotify   destroy);
+				    GDestroyNotify     destroy);
 void	   gtk_quit_remove	   (guint	       quit_handler_id);
 void	   gtk_quit_remove_by_data (gpointer	       data);
-#ifndef GTK_DISABLE_DEPRECATED
 guint	   gtk_timeout_add	   (guint32	       interval,
 				    GtkFunction	       function,
 				    gpointer	       data);
@@ -176,7 +178,7 @@ guint	   gtk_timeout_add_full	   (guint32	       interval,
 				    GtkFunction	       function,
 				    GtkCallbackMarshal marshal,
 				    gpointer	       data,
-				    GtkDestroyNotify   destroy);
+				    GDestroyNotify     destroy);
 void	   gtk_timeout_remove	   (guint	       timeout_handler_id);
 
 guint	   gtk_idle_add		   (GtkFunction	       function,
@@ -188,7 +190,7 @@ guint	   gtk_idle_add_full	   (gint	       priority,
 				    GtkFunction	       function,
 				    GtkCallbackMarshal marshal,
 				    gpointer	       data,
-				    GtkDestroyNotify   destroy);
+				    GDestroyNotify     destroy);
 void	   gtk_idle_remove	   (guint	       idle_handler_id);
 void	   gtk_idle_remove_by_data (gpointer	       data);
 guint	   gtk_input_add_full	   (gint	       source,
@@ -196,7 +198,7 @@ guint	   gtk_input_add_full	   (gint	       source,
 				    GdkInputFunction   function,
 				    GtkCallbackMarshal marshal,
 				    gpointer	       data,
-				    GtkDestroyNotify   destroy);
+				    GDestroyNotify     destroy);
 void	   gtk_input_remove	   (guint	       input_handler_id);
 #endif /* GTK_DISABLE_DEPRECATED */
 
@@ -223,9 +225,6 @@ gboolean _gtk_boolean_handled_accumulator (GSignalInvocationHint *ihint,
 
 gchar *_gtk_get_lc_ctype (void);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
-
+G_END_DECLS
 
 #endif /* __GTK_MAIN_H__ */
